use std::time::Duration;

use crate::repositories;
use diesel::{
  r2d2::{ConnectionManager, Pool},
  PgConnection,
};
use tokio_cron_scheduler::{Job, JobScheduler};

///
pub struct Scheduler {
  pool: Pool<ConnectionManager<PgConnection>>,
  sched: JobScheduler,
  vendor_repository: repositories::vendor::repository::Repository,
}

impl std::fmt::Debug for Scheduler {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Scheduler of api.kgon.identity").finish()
  }
}

impl Scheduler {
  ///
  pub fn new(pool: Pool<ConnectionManager<PgConnection>>, sched: JobScheduler) -> Scheduler {
    Scheduler {
      pool,
      sched,
      vendor_repository: repositories::vendor::repository::Repository::new(),
    }
  }

  pub async fn queue(&self) -> Result<(), std::boxed::Box<dyn std::error::Error>> {
    self.list_members().await?;

    Ok(())
  }

  async fn list_members(&self) -> Result<(), Box<dyn std::error::Error>> {
    let mut jj = Job::new_repeated(Duration::from_secs(60), |_uuid, _l| {
      println!("{:?} I'm repeated every 60 seconds", chrono::Utc::now());
    })
    .unwrap();

    jj.on_start_notification_add(
      &self.sched,
      Box::new(|job_id, notification_id, type_of_notification| {
        Box::pin(async move {
          println!(
            "Job {:?} was started, notification {:?} ran ({:?})",
            job_id, notification_id, type_of_notification
          );
        })
      }),
    )
    .await;

    jj.on_stop_notification_add(
      &self.sched,
      Box::new(|job_id, notification_id, type_of_notification| {
        Box::pin(async move {
          println!(
            "Job {:?} was completed, notification {:?} ran ({:?})",
            job_id, notification_id, type_of_notification
          );
        })
      }),
    )
    .await;

    jj.on_removed_notification_add(
      &self.sched,
      Box::new(|job_id, notification_id, type_of_notification| {
        Box::pin(async move {
          println!(
            "Job {:?} was removed, notification {:?} ran ({:?})",
            job_id, notification_id, type_of_notification
          );
        })
      }),
    )
    .await;
    self.sched.add(jj).await;

    Ok(())
  }
}
